using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Request.Courses;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.Request;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 教培服务接口
    /// </summary>
    public interface IEducationService
    {
        /// <summary>
        /// 获取标签列表 
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>标签信息</returns>
        /// <errorCodes></errorCodes>
        Task<IList<CourseLabelDTO>> GetCourseLabelsAsync(QueryCourseLabelListRequest request);

        /// <summary>
        /// 申请课程
        /// </summary>
        /// <param name="request">申请课程请求</param>
        /// <returns>成功或者失败</returns>
        /// <errorCodes>TeacherCodeEmpty,TeacherNoFound,UserNotFind </errorCodes>
        Task<bool> ApplyCourseAsync(ApplyCourseRequest request);

        /// <summary>
        /// 查询自己申请课程列表分页
        /// </summary>
        /// <param name="request">查询自己申请课程请求</param>
        /// <returns>课程列表信息</returns>
        ///  <errorCodes>UserNotFind</errorCodes>
        Task<PageResult<CoursePageDTO>> FindCoursePagesAsync(FindCoursePagesRequest request);

        /// <summary>
        /// 删除课程记录
        /// </summary>
        /// <param name="request">删除课程请求</param>
        /// <returns>删除成功或者失败</returns>
        /// <errorCodes>UserNotFind,CourseCodeEmpty,CourseNoFound,NotDeleteForCourseStatus</errorCodes>
        Task<bool> DeleteCourseByCodeAsync(DeleteCourseByCodeRequest request);

        /// <summary>
        /// 更新课程记录
        /// </summary>
        /// <param name="request">更新课程请求</param>
        /// <returns>更新成功或者失败</returns>
        ///  <errorCodes>UserNotFind,CourseCodeEmpty,TeacherNoFound,CourseNoFound,NotUpdateForCourseStatus </errorCodes>
        Task<bool> UpdateCourseAsync(UpdateCourseRequest request);

        /// <summary>
        /// 根据课程编码获取课程详细信息
        /// </summary>
        /// <param name="request">获取详细信息课程请求</param>
        /// <returns>课程详细信息</returns>
        ///  <errorCodes>UserNotFind,CourseCodeEmpty</errorCodes>
        Task<CourseInfoDetailDTO> FindCourseByCodeAsync(FindCourseByCodeRequest request);

        /// <summary>
        /// 查询自己当学生的课程列表分页
        /// </summary>
        /// <param name="request">学生查询自己课程请求</param>
        /// <returns>课程列表</returns>
        /// <errorCodes>UserNotFind</errorCodes>
        Task<PageResult<StudentInCoursePageDTO>> FindStudentInCoursePagesAsync(FindStudentInCoursePagesRequest request);

        /// <summary>
        /// 客户端根据创建人获取创建的用户组
        /// </summary> 
        /// <param name="request">创建人code</param>
        /// <returns></returns>
        /// <errorCodes>UserNotFind</errorCodes>
        Task<IList<UserGroupDTO>> GetUserGroupListsByCreatorCodeAsync(TokenRequest request); 

        /// <summary>
        /// 新增用户组
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        /// <errorCodes>UserNotFind,UserGroupNameIsEmpty,UserGroupMaxPeopleNotValid</errorCodes>
        Task<bool> AddUserGroupAsync(UserGroupRequest request);

        /// <summary>
        /// 查询用户分页
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        /// <errorCodes>UserGroupCodeIsEmpty</errorCodes>
        Task<PageResult<StudentInfoDTO>> FindUserGroupStudentsPagesAsync(QueryStudentByGroupCodePageRequest request);

        /// <summary>
        /// 用户组移除用户
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        /// <errorCodes>UserGroupCodeIsEmpty,UserGroupStudentsIsEmpty</errorCodes>
        Task<bool> RemoveStudentsByGroupCodeAsync(UserGroupFilterRequest request);

        /// <summary>
        /// 新增试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AddExaminationPaperAsync(AddExaminationPaperRequest request);

        /// <summary>
        /// 删除试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeleteExaminationPaperAsync(DeleteExaminationPaperRequest request);

        /// <summary>
        /// 修改试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UpdateExaminationPaperAsync(UpdateExaminationPaperRequest request);

        /// <summary>
        /// 查询试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<ExaminationPaperDTO> GetExaminationPaperAsync(GetExaminationPaperRequest request);

        /// <summary>
        /// 查询试卷分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<ExaminationPaperDTO>> GetExaminationPaperPagesAsync(GetExaminationPaperPagesRequest request);
    }
}